#include <stdio.h>
#include <math.h>
#include <unistd.h>

#include "err.h"

#define ICOUNT_PATH "/dev/icount"
#define ICOUNT_CONF_PATH "/usr/share/icount.conf"

typedef unsigned long long ULL;

typedef struct {
    double a, b, bias, epsilon;
} icount_t;

double max(double a, double b) {
    return (a > b) ? a : b;
}

double get_pred(icount_t *ic, double x) {
    return exp(ic->a * log(max(ic->epsilon, x - ic->bias)) + ic->b);
}

void read_icount(ULL *ts, ULL *tc) {
    FILE *ifile;
    SYS_FAIL_IF(!(ifile = fopen(ICOUNT_PATH, "r")));
    SYS_FAIL_IF(fscanf(ifile, "%llu %llu", ts, tc) != 2);
    SYS_FAIL_IF(fclose(ifile) != 0);
}

void read_icount_conf(icount_t *ic) {
    FILE *cfile;
    SYS_FAIL_IF(!(cfile = fopen(ICOUNT_CONF_PATH, "r")));
    SYS_FAIL_IF(fscanf(cfile, "(%lf, %lf, %lf, %lf)",
                       &ic->a, &ic->b, &ic->bias, &ic->epsilon) != 4);
    SYS_FAIL_IF(fclose(cfile) != 0);
}

double icount_measure(unsigned duration_s) {
    ULL sup_ts[2];
    ULL sup_tc[2];
    read_icount(&sup_ts[0], &sup_tc[0]);
    sleep(duration_s);
    read_icount(&sup_ts[1], &sup_tc[1]);
    return (double)(sup_tc[1] - sup_tc[0]) / (sup_ts[1] - sup_ts[0]);
}

int main(int argc, char ** argv) {
    unsigned duration_s;
    icount_t ic;
    double measured_val;
    double result;
    int get_only_measurement;

    if (argc < 2) {
        fprintf(stderr, "Usage: %s <measurement_duration_in_s> [--no-cal]\n", argv[0]);
        return 1;
    }

    if (sscanf(argv[1], "%d", &duration_s) < 1 || duration_s == 0) {
        fprintf(stderr, "Duration parsing error - it should be a positive int\n");
        return 1;
    }
        measured_val = icount_measure(duration_s);

    get_only_measurement = argc >= 3 && strcmp(argv[2], "--no-cal") == 0;

    if (get_only_measurement) {
        result = icount_measure(duration_s);
    }
    else {
        read_icount_conf(&ic);
        measured_val = icount_measure(duration_s);
        result = get_pred(&ic, measured_val);
    }
    printf("%.17g\n", result);
    return 0;
}
