#ifndef _ERR_H__
#define _ERR_H__

#include <stdio.h>
#include <errno.h>
#include <string.h>

// We'll use stdout here because otherwise the order of printfs is
// undeterministic which can be sometimes confusing
#define LOG(name, err) \
    fprintf(stdout, "File %s, line %d, '%s' failed with errno=(%d, %s)\n", \
             __FILE__,__LINE__, name, err, strerror(err))

#define SYSERR(name, err) {\
    LOG(name, err); \
    exit(1); \
}

#define THREAD_SYSERR(name, err) {\
    LOG(name, err); \
    pthread_exit((void*)1); \
}

#define THREAD_FATAL(msg) THREAD_SYSERR(msg, 0)

#define SYS_FAIL_IF(cond) if ((cond)) SYSERR((#cond), errno)

#define SYS_FAIL_IF_ERR(cond, err) if ((cond)) SYSERR((#cond), err)

#define THR_FAIL_IF(cond) if ((cond)) THREAD_SYSERR((#cond), errno)

#define THR_GOTO_IF(cond, label) if ((cond)) {\
    LOG((#cond), errno); \
    goto label; \
}

#endif
