// Definitions for PCF8593 low power clock and calendar
// See http://www.nxp.com/documents/data_sheet/PCF8593.pdf
#ifndef _PCH8593_H
#define _PCH8593_H

#include<linux/types.h>

// Registers
const u8 PCF8593_CONTROL       = 0x00;
const u8 PCF8593_D1_D0         = 0x01;
const u8 PCF8593_D3_D2         = 0x02;
const u8 PCF8593_D5_D4         = 0x03;
const u8 PCF8593_TIMER         = 0x07;
const u8 PCF8593_ALARM         = 0x08;

// Values
const u8 PCF8593_CONTROL_MODE_EVENT_COUNTER = (1<<5) | (0<<4);
const u8 PCF8593_CONTROL_ALARM_ENABLE       = (1<<2);

const u8 PCF8593_ALARM_TIMER_MODE_DISABLED        = 0;
const u8 PCF8593_ALARM_TIMER_MODE_1               = (1<<0);
const u8 PCF8593_ALARM_TIMER_MODE_100             = (1<<1);
const u8 PCF8593_ALARM_TIMER_MODE_10k             = (1<<1) | (1<<0);
const u8 PCF8593_ALARM_TIMER_MODE_1M              = (1<<2);
const u8 PCF8593_ALARM_TIMER_MODE_TEST            = (1<<2) | (1<<1) | (1<<0);

const u8 PCF8593_ALARM_TIMER_INTERRUPT            = (1<<3);
const u8 PCF8593_ALARM_TIMER_ALARM                = (1<<6);
const u8 PCF8593_ALARM_ALARM_INTERRUPT            = (1<<7);

#define pcf8593_combine_2_digits(d1d0) ((d1d0 & 0x0f) + 10*((d1d0 >> 4) & 0x0f))
#define pcf8593_combine_8_digits(d7d6, d5d4, d3d2, d1d0) \
  (pcf8593_combine_2_digits(d1d0) + \
  pcf8593_combine_2_digits(d3d2) * 100 + \
  pcf8593_combine_2_digits(d5d4) * 100 * 100 + \
  pcf8593_combine_2_digits(d7d6) * 100 * 100 * 100)

// Counter counts from 0 to PCF8593_OVERFLOW-1
const long long PCF8593_OVERFLOW = (pcf8593_combine_8_digits(0x99, 0x99, 0x99, 0x99) + 1);

#endif
