/*
 * buffer_monitor.h
 *
 *  Created on: Feb 11, 2017
 *      Author: P. Horban <p.horban@mimuw.edu.pl>
 */

#ifndef BUFFER_MONITOR_H_
#define BUFFER_MONITOR_H_

#include "com.pb-c.h"

#include "common.h"

/** All functions are thread-safe unless noted otherwise */

/**
 * Returns 0 on success. Not thread safe.
 * The module will not use more than max_memory_use + O(1) memory.
 */
int bm_init(size_t _max_memory_use);

/**
 * Allows to change the memory limit size. The change may not be immediate.
 */
void bm_set_max_memory_use(size_t max_memory_use);

/**
 * Fills line related fields of to_server message. Returns 0 on success.
 * Use bm_free_lines to deallocate memory before destroying rest of the
 * pb message.
 *
 * Fields set by this call:
 * 	- min_available_line_seq_nr
 * 	- min_available_line_ts_ms
 * 	- first_line_seq_nr
 * 	- lines
 *
 * If no lines are available, the function will block. If the req_line_seq_nr is
 * smaller than lowest available line number, the function will start at lowest
 * available line. If req_line_seq_nr is one above highest available line, the
 * function will block waiting for data. If it is higher, the an EINVAL error
 * will be returned. You can pass a timeout value. Then the function returns
 * ETIMEDOUT if that timeout is reached. Currenly you must set a timeout > 0.
 *
 * 	max_lines_size must be >= 2 * LINE_SIZE_LIMIT
 */
int bm_copy_lines(ToServer *to_server,  line_seq_nr_t req_line_seq_nr,
				  size_t max_lines_size, size_t max_wait_sec);

/** Releases memory allocated by bm_copy_lines */
void bm_free_lines(ToServer *to_server);

/**
 * Adds a line to the end of the buffer. Copies it internally, without
 * taking ownership of the line.
 */
void bm_append_line(timestamp_t unix_ts_ms, char *line); // todo, you cant send 0, fileter any 0s out.

/**
 * Not thread safe. Should be called by the creating thread when all
 * other threads stopped using the monitor
 */
void bm_destory();


#endif /* BUFFER_MONITOR_H_ */
