#ifndef CLICOM_H_
#define CLICOM_H_

#include "com.pb-c.h"

/**
 * Initialises the library.
 * Returns 0 on success.
 */
int com_init(char *id_str, const char *ip_addr, int port);

/**
 * You should always call com_next_msg_to_server to obtain a new ToServer.
 * Fill it in and then call com_communicate which will free the ToServer object.
 * Then process the response and free it's resources with release_response.
 *
 * On success com_next_msg_to_server will return 0, and msg_to_server will
 * have the address of the new message object.
 * In case of failure, msg_to_server is undefined and can't be used with
 * com_communicate.
 * ToServer must always be released after com_communicate.
 *
 * On success com_communicate will return 0 and set msg_to_client to have
 * the address of the response object.
 * On failure msg_to_client will be undefined and can't be used with
 * release_response.
 */
int com_next_msg_to_server(ToServer **msg_to_server);
int com_communicate(ToServer *msg_to_server, ToClient **msg_to_client,
					int recv_wait_ms, int *actual_recv_wait_ms);
void com_release_to_server(ToServer **msg_to_server);
void com_release_to_client(ToClient **msg_to_client);
int com_get_session_id();
void com_destroy();


#endif /* CLICOM_H_ */
