/*
 * line_size_limit.h
 *
 *  Created on: Feb 14, 2017
 *      Author: P. Horban <p.horban@mimuw.edu.pl>
 */

#ifndef COMMON_H_
#define COMMON_H_

#include <stdio.h>
#include <stdint.h>

#define MAX_MEMORY_USE

/**
 * This should be interpreted as maximum number of characters that a line can
 * contain. This does not include the terminating 0 byte. Therefore buffers
 * should use the BUFFER_SIZE_LIMIT constant.
 */
#define LINE_SIZE_LIMIT 250
#define BUFFER_SIZE_LIMIT (LINE_SIZE_LIMIT + 1)

// Unix UTC timestamp in ms since Epoch.
typedef long long timestamp_t;
typedef int64_t line_seq_nr_t;

line_seq_nr_t max(line_seq_nr_t a, line_seq_nr_t b);
line_seq_nr_t min(line_seq_nr_t a, line_seq_nr_t b);

enum {
	OFF = 0,
	FATAL = 1,
	ERROR = 2,
	WARN = 3,
	INFO = 4,
	DEBUG = 5,
	TRACE = 6,
	TRACE2 = 7,
	TRACE3 = 8,
};

#define log_level WARN

const char *_lvl_to_str(size_t lvl);

#define LOG(lvl, msg) \
	if (lvl <= log_level) { \
     	fprintf(stderr, "%s " msg "\n", _lvl_to_str(lvl)); }

#define LOGV(lvl, msg, ...) \
	if (lvl <= log_level) { \
     	fprintf(stderr, "%s " msg "\n", _lvl_to_str(lvl), __VA_ARGS__); }

#define LOG_SE(lvl, msg) \
	if (lvl <= log_level) { \
     	fprintf(stderr, "%s " msg "\n", _lvl_to_str(lvl)); \
     	perror("     Cause"); }

#endif /* COMMON_H_ */
