/*
 * config_file.c
 *
 *  Created on: Feb 20, 2017
 *      Author: P. Horban <p.horban@mimuw.edu.pl>
 */

#include <string.h>
#include <assert.h>

#include "config_file.h"

#include <libconfig.h>

#include "common.h"


#define LOAD_TC_SIZE_T(name) \
	{ long long val; \
	if (config_lookup_int64(&config, #name, \
			&(val)) != CONFIG_TRUE || val < 0) { \
			LOG(WARN, "Could not read config " #name ". Is it >= 0?"); \
			config_destroy(&config); \
			return -1; \
	} else { app_config->linetc_conf.name = (size_t)val; }  }

#define LOAD_TC_DOUBLE(name) \
	if (config_lookup_float(&config, #name, \
			&(app_config->linetc_conf.name)) != CONFIG_TRUE) { \
			LOG(WARN, "Could not read config " #name "."); \
			config_destroy(&config); \
			return -1; }

int read_config_file(char *file_path, app_config_t *app_config) {
	config_t config;
	config_init(&config);
	if (config_read_file(&config, file_path) == CONFIG_TRUE) {
		long long val;

		if (config_lookup_int64(&config, "max_memory_use", &(val)) !=
				CONFIG_TRUE || val <= 0) {
				LOG(WARN, "Could not read config max_memory_use. Is it >= 0?");
				config_destroy(&config);
				return -1;
		} else
			app_config->max_memory_use = (size_t)val;

		const char *ip;
		if (config_lookup_string(&config, "server_ip", &ip) != CONFIG_TRUE) {
			LOG(WARN, "Could not read server_ip");
			config_destroy(&config);
			return -1;
		}

		if (strlen(ip) >= 50) {
			LOG(WARN, "server_ip field too long. Must be < 50 chars.");
			config_destroy(&config);
			return -1;
		}

		memcpy(app_config->server_ip, ip, strlen(ip) + 1);

		if (config_lookup_int64(&config, "server_port", &(val)) !=
				CONFIG_TRUE || val <= 0 || val >= (1<<16)) {
				LOG(WARN, "Could not read config server_port. "
						"Is it and uint16_t?");
				config_destroy(&config);
				return -1;
		} else
			app_config->server_port = (uint16_t)val;


		LOAD_TC_SIZE_T(min_packet_size);
		LOAD_TC_SIZE_T(max_packet_size);
		LOAD_TC_SIZE_T(packet_size_increment);
		LOAD_TC_SIZE_T(min_recv_wait_ms);
		LOAD_TC_SIZE_T(max_recv_wait_ms);
		LOAD_TC_SIZE_T(keep_alive_max_delay_sec);
		LOAD_TC_DOUBLE(packet_shrink_multiplier);
		LOAD_TC_DOUBLE(recv_wait_avg_keep_coef);
		LOAD_TC_DOUBLE(recv_wait_avg_componet);
		LOAD_TC_DOUBLE(recv_wait_const_component);
	}
	else {
		LOG(ERROR, "Could not open/parse the config file:");
		fprintf(stderr, "%s:%s:%d\n", config_error_text(&config),
				config_error_file(&config), config_error_line(&config));
		config_destroy(&config);
		return -1;
	}

	config_destroy(&config);
	return 0;
}
