/*
 * linetc.h
 *
 *  Created on: Feb 10, 2017
 *      Author: P. Horban <p.horban@mimuw.edu.pl>
 */

/**
 * Zarzadzanie transmisja:
 *
 * Rozmiar pakietu w przedziale [500B, 20kB]
 * Rozmiar rośnie o 250 bajtow gdy odbierzemy odpowiedz, ale spada dwukrotnie
 * gdy nie obierzemy odpowiedzi.
 *
 * Przy czasie obrotu 60ms, maksymalna predkosc transmisji to 333 kB/s.
 *
 * Czas timeoutu pakietu zawiera sie w przedziale [10ms, 20s].
 * Liczony jest jako a*MT + b, gdzie
 * MT = srednia dlugosc oczekiwania na odpowiedz, liczona srednią wykladnicza:
 * avg = avg*c + (1.0-c)*wait_time
 * a, b, c - konfigurowalne parametry
 * W przypadku braku odpowiedzi, wait_time liczymy jako caly czas oczekiwania,
 * czyli do średniej dodaje się wait time pomnozony przez 2, wiec czas
 * oczekiwania szybko rosnie.
 *
 * Future Work: Mozna to dalej przyspieszy wysylajac kilka pakietow, bez
 * czekania na odpowiedz z serwera - czyli tx tx tx tx rx tx tx tx tx rx. To
 * lepiej wysyci pipeline i zwiekszy predkosc transmisji nawet do kilku MB/s.
 *
 * Po kazdym pakiecie linetc wola bm_wait_for_data, ktora konczy sie natychmiast
 * jest jesli dane sa dostepne, lub po okresie timeout jesli dane nie sa
 * dostepen Potem linetc i tak wysyla pusty pakiet, jako forme keep-alive z
 * serwerem. Czestotliwosc takich pakietow mala - 1/20s.
 */

#ifndef LINETC_H_
#define LINETC_H_

#include <stdlib.h>

typedef struct  {
	/**
	 * Packets have a range of possible sizes. The size grows linearly by
	 * increment, but shrinks exponentially by multplying by the shrink multipilier.
	 */
	size_t min_packet_size;
	size_t max_packet_size;
	size_t packet_size_increment;
	double packet_shrink_multiplier;

	/** This would be 0.9 according to description above. An exp. avg. */
	double recv_wait_avg_keep_coef;

	/**
	 * Packet receive wait time can take a range of values. It is calculated
	 * based on the avg. wait time.
	 */
	double recv_wait_avg_componet;
	double recv_wait_const_component;
	size_t min_recv_wait_ms;
	size_t max_recv_wait_ms;

	/** Max delay for sending a possibly empty keep-alive packet. */
	size_t keep_alive_max_delay_sec;
} linetc_conf_t;

/* Initializes the linetc module */
int linetc_init(char *id_str, const char *ip_addr, int port,
		linetc_conf_t conf);

/* Updates the linetc config */
int linetc_set_config(linetc_conf_t conf);

/* Handles single packet transmission. Should be called in a loop. */
void linetc_handle_one_packet(void);

/* Free resources used by linetc module */
void linetc_destory();


#endif /* LINETC_H_ */

// todo, pewne parametry konfiguracyjne powinny być zmienialne
