/*
 * ttyread.c
 *
 *  Created on: Feb 16, 2017
 *      Author: P. Horban <p.horban@mimuw.edu.pl>
 */


#include <errno.h>
#include <errno.h>
#include <fcntl.h>
#include <poll.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

#include "common.h"
#include "buffer_monitor.h"

static int POLL_INTERVAL_MS = 1000;

static char *file_path = NULL;
static struct pollfd fds[1] = {{-1, 0, 0}};
static int stopped = 1;

static timestamp_t get_unix_utc_ts();
static void append_byte(char *arr, size_t *arr_len, char b);
static int open_fd(int keep_trying);

int tr_init(char *_file_path) {
	stopped = 0;
	file_path = _file_path;
	return errno = 0;
}

void tr_destroy() {
	stopped = 1;
	if (fds[0].fd != -1) {
		close(fds[0].fd);
		fds[0].fd = -1;
	}
	file_path = NULL;
}

void tr_main_loop() {
	size_t lbuf_len = 0;
	char lbuf[BUFFER_SIZE_LIMIT];
	char rbuf[BUFFER_SIZE_LIMIT];

	stopped = 0;
	open_fd(1);
	while(!stopped) {
		int pollres = poll(fds, 1, POLL_INTERVAL_MS);
		if (pollres == 1) {
			ssize_t n = read(fds[0].fd, rbuf, LINE_SIZE_LIMIT);

			if (n == -1 || n == 0) {
				if (open_fd(1) != 0) {
					if (stopped)
						break;
					else {
						fprintf(stderr, "FATAL: open_fd failed and returned "
								"- that should not be possible.\n");
						exit(-1);
					}
				}
			}


			for (int i = 0; i < n; i++) {
				if (rbuf[i] == '\n') {
					lbuf[lbuf_len] = 0;
					LOGV(TRACE2, "Appending line with %zu bytes", lbuf_len);
					bm_append_line(get_unix_utc_ts(), lbuf);
					lbuf_len = 0;
				}
				else if(rbuf[i] == 0) {
					// Mask invalid 0 character.
					append_byte(lbuf, &lbuf_len, '\\');
					append_byte(lbuf, &lbuf_len, '0');
				}
				else {
					append_byte(lbuf, &lbuf_len, rbuf[i]);
				}
			}



		}
		else if(pollres == 0) {
			// Timeout. Used so that the while has a chance to stop.
		}
		else {
			if (open_fd(1) != 0) {
				if (stopped)
					break;
				else {
					fprintf(stderr, "FATAL: open_fd failed and returned "
							"- that should not be possible.\n");
					exit(-1);
				}
			}
		}
	}
}

void tr_stop_main() {
	stopped = 1;
}

static timestamp_t get_unix_utc_ts() {
    struct timespec t;

    if( clock_gettime( CLOCK_REALTIME, &t) == -1 ) {
      perror( "clock_gettime( CLOCK_REALTIME, &start)" );
      exit( -1 );
    }

    timestamp_t ts = 1000;
    ts *= t.tv_sec;
    ts += t.tv_nsec / 1000000;
    return ts;
}

static void append_byte(char *buf, size_t *buf_len, char b) {
	static char trunc_str[] = " [trunc]";
	static size_t trunc_len = sizeof(trunc_str) - 1;

	if (*buf_len == LINE_SIZE_LIMIT)
		return;
	else if(*buf_len == LINE_SIZE_LIMIT-1) {
		// od 0 ost bajt trafi na buf[trunc_len-1]
		// chcemy ost bajt na buf[LINE_SIZE_LIMIT-1]
		// trunc_len - 1 + x = LINE_SIZE_LIMIT-1
		// x = LINE_SIZE_LIMIT - trunc_len
		size_t x = LINE_SIZE_LIMIT - trunc_len;
		memcpy(buf + x, trunc_str, trunc_len);
		*buf_len = LINE_SIZE_LIMIT;
	}
	else {
		buf[*buf_len] = b;
		(*buf_len) += 1;
	}
}

static int open_fd(int keep_trying) {
	if (fds[0].fd != -1) {
		close(fds[0].fd);
		fds[0].fd = -1;
	}

	fds[0].events = POLLIN | POLLPRI; // check for normal or priority read

	while (1) {
		fds[0].fd = open(file_path, O_RDONLY | O_NONBLOCK);
		if (fds[0].fd != -1)
			break;
		if (stopped || !keep_trying)
			break;
		sleep(1);
	}

	if (fds[0].fd == -1) {
		LOG_SE(ERROR, "Failed to open file");
		return errno;
	}
	else
		return 0;
}
